(***************************************************************************)
(* from Pascal Courte-Heuse  CIS : 101667.455                              *)
(*                                                                         *)
(* WARNING, previous version is ILLEGAL, you must use this one             *)
(* Purpose : MessageID must be between $0000 and $FFFF value               *)
(* but Microsoft said : (found in Windows SDK description file)

WM_USER is a constant used by applications to help define private messages.
Comments

The WM_USER constant is used to distinguish between message values that are
reserved for use by Windows and values that can be used by an application
to send messages within a private window class. There are four ranges of
message numbers:

Range	Meaning

0 through WM_USER - 1	Messages reserved for use by Windows.
WM_USER through 0x7FFF	Integer messages for use by private window classes.
0x8000 through 0xBFFF	Messages reserved for use by Windows.
0xC000 through 0xFFFF	String messages for use by applications.

Message numbers in the first range (0 through WM_USER - 1) are defined by
Windows. Values in this range that are not explicitly defined are reserved
for future use by Windows. This topic describes messages in this range.
Message numbers in the second range (WM_USER through 0x7FFF) can be defined
and used by an application to send messages within a private window class.
These values cannot be used to define messages that are meaningful throughout
an application, because some predefined window classes already define values
in this range. For example, such predefined control classes as BUTTON, EDIT,
LISTBOX, and COMBOBOX may use these values. Messages in this range should not
be sent to other applications unless the applications have been designed to
exchange messages and to attach the same meaning to the message numbers.
Message numbers in the third range (0x8000 through 0xBFFF) are reserved for
future use by Windows.

Message numbers in the fourth range (0xC000 through 0xFFFF) are defined at
run time when an application calls the RegisterWindowMessage function to
obtain a message number for a string. All applications that register the same
string can use the associated message number for exchanging messages.
The actual message number, however, is not a constant and cannot be
assumed to be the same in different Windows sessions.

RegisterWindowMessage is typically used to register messages for
communicating between two cooperating applications.

If two different applications register the same message string,
the applications return the same message value. The message remains
registered until the Windows session ends.
Use the RegisterWindowMessage function only when more than one application
must process the same message. For sending private messages within a window
class, an application can use any integer in the range WM_USER through 0x7FFF.
(Messages in this range are private to a window class, not to an application.
For example, such predefined control classes as BUTTON, EDIT,
LISTBOX, and COMBOBOX may use values in this range.)

The following show an implementation for an efficient exchange message between
many applications.
Note than code is not very different, I have just made change in constructor 
*)

(* original comments *)
{This unit shows how to accomplish program to program communication
 through message passing. The trick is to create a custom message that
 only your apps can send and will respond to.

 To demonstrate, compile the project to an exe, then start 2 copies
 of the program. When the "Get Data" button is pressed, the other app will
 receive the message and send back a message which points to the data
 required. In this example the sender sends his specific window handle, so
 the responder can send the message only to him.

 I had a need to do this sort of thing for a project, but Delphi Tech pay
 for Support was no help, so this was a trial and error project}

unit Getdata1;

{ note : i have removed all old comment and add the new ones for more clearly }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls;

Const
  { space for auto-num MessageID }
  { if allocation is aborted, then a run-time error (202) is occur }
  { to prevent it, change the initial values with something else  }
  RequestMsgID : Word = 0;
  ResponseMsgID : Word = 0;


  { real constants, the name of events. Windows store full name in is stack }
  txtRequestMsgID  = 'RequestMsgID';
  txtResponseMsgID = 'ResponseMsgID';


type
  TCustomMsg = Record
    Msg    : Cardinal;
    Sender : Word;
    Data   : Longint;
    Result : Longint;
  end;

  TDataRec = Class
    EffDate     : TdateTime;
    ExpDate     : TDateTime;
    BadgeNumber : Integer;
    UserID      : String;
  End;

  TForm1 = class(TForm)
    Edit1: TEdit;
    Button1: TButton;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Edit6: TEdit;
    Edit7: TEdit;
    Edit8: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    { overwrite the default message respond procedure }
    procedure DefaultHandler(var aMessage);override;
    { build the procedure called by DefaultHandler ( better way instead write
      all code in DefaultHandler) }
    Procedure RespondToRequest(Var Msg:TCustomMsg);
    Procedure SendData(HWindow:Hwnd);
    Procedure ReceiveData(Var Msg:TCustomMsg);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation
Var DataRec:TDataRec;
{$R *.DFM}

{ Default Handler proc, look for event, take effect if it is specific one
  call the inherited proc if none }
procedure TForm1.DefaultHandler(var aMessage);
begin
  with tMessage(aMessage) Do
    { it is not possible to use a case statment, the value of xxxMsgID is
      know only at run time. When message is identify, then call the right proc}
    if Msg = RequestMsgID Then SendData(TCustomMsg(AMessage).Sender)
    Else if Msg = ResponseMsgID Then receiveData(TCustomMsg(AMessage))
    { not a specific message, run standard proc }
    Else inherited DefaultHandler(aMessage);
end;

Procedure TForm1.SendData(HWindow:Hwnd);
Begin
  With DataRec do
  Begin
    EffDate := StrToDate(Edit1.Text);
    ExpDate := StrToDate(Edit2.Text);
    BadgeNumber := StrToInt(Edit3.Text);
    UserID := Edit4.Text;
  End;
  { send message to caller }
  SendMessage(HWindow,ResponseMsgID,Handle,LongInt(DataRec));
End;


Procedure TForm1.RespondToRequest(Var Msg:TCustomMsg);
Begin
  {Request message is received here, the sender field is the handle of
   The requester}
  SendData(Msg.Sender);
End;


procedure TForm1.Button1Click(Sender: TObject);
Var s:String;
begin
  { send message to Windows Desktop }
  SendMessage(hwnd_Broadcast,RequestMsgID,Handle,0);
end;

Procedure TForm1.ReceiveData(Var Msg:TCustomMsg);
Begin
  MessageBeep(0);
  if not (Handle = Msg.Sender) Then Begin
    Label2.Caption := 'Message Received at '+TimeToStr(Time)+ ' by Window #'+IntToStr(Msg.Sender);
    Try
      Edit5.Text := DateToStr(TDataRec(msg.Data).EffDate);;
      Edit6.Text := DateToStr(TDataRec(msg.Data).ExpDate);
      Edit7.Text := IntToStr(TDataRec(msg.Data).BadgeNumber);
      Edit8.Text := TDataRec(msg.Data).UserID;
    Except on E : EConvertError do
      ShowMessage(E.Message);
    End;
  End;
End;

procedure TForm1.FormCreate(Sender: TObject);
Var strTemp : Array[0..20] of Char;
begin
  DataRec := TDataRec.Create;

   { get Valid MessageID }
   strPCopy(strTemp,txtResponseMsgID);
   ResponseMsgID := RegisterWindowMessage(strTemp);
   strPCopy(strTemp,txtRequestMsgID);
   RequestMsgID := RegisterWindowMessage(strTemp);

  Edit1.Text := DateToStr(Date);
  Edit2.Text := DateToStr(Date+360);
  Edit3.Text := '100';
  Edit4.Text := IntToStr(Handle);
  Form1.Caption := 'Window #'+IntToStr(Handle);
  end;

end.

{
 No change, no code in Project source ... ;-)
}

